@@ -43,6 +43,20 @@ SKIP_BASE_INSTALL_ARG = click.option(
 PYTHON_VERSIONS_ARG = click.option(
     "-p", "--python", "pythons", type=InterpreterParamType(), default=[], multiple=True
 )
+INTERPRETERS_ARG = click.option(
+    "-i",
+    "--interpreters",
+    "interpreters",
+    is_flag=True,
+    default=False,
+)
+RECOMPILE_REQS_ARG = click.option(
+    "-c",
+    "--recompile-requirements",
+    "recompile_reqs",
+    is_flag=True,
+    default=False,
+)
 
 
 @click.group()
@@ -91,13 +105,23 @@ def main(ctx, riotfile, log_level, pipe_mode):
 @PYTHON_VERSIONS_ARG
 @PATTERN_ARG
 @VENV_PATTERN_ARG
+@INTERPRETERS_ARG
+@click.option(
+    "--hash-only",
+    "hash_only",
+    is_flag=True,
+    default=False,
+    help="Only print the hashes of matched venvs",
+)
 @click.pass_context
-def list_venvs(ctx, pythons, pattern, venv_pattern):
+def list_venvs(ctx, pythons, pattern, venv_pattern, interpreters, hash_only):
     ctx.obj["session"].list_venvs(
         re.compile(pattern),
         re.compile(venv_pattern),
         pythons=pythons,
         pipe_mode=ctx.obj["pipe"],
+        interpreters=interpreters,
+        hash_only=hash_only,
     )
 
 
@@ -139,6 +163,7 @@ def generate(ctx, recreate_venvs, skip_base_install, pythons, pattern):
 @click.option("--exitfirst", "-x", "exit_first", is_flag=True, default=False)
 @PATTERN_ARG
 @VENV_PATTERN_ARG
+@RECOMPILE_REQS_ARG
 @click.pass_context
 def run(
     ctx,
@@ -150,6 +175,7 @@ def run(
     exit_first,
     pattern,
     venv_pattern,
+    recompile_reqs,
 ):
     ctx.obj["session"].run(
         pattern=re.compile(pattern),
@@ -161,6 +187,7 @@ def run(
         pythons=pythons,
         skip_missing=skip_missing,
         exit_first=exit_first,
+        recompile_reqs=recompile_reqs,
     )
 
 
@@ -173,3 +200,12 @@ def shell(ctx, ident, pass_env):
         ident=ident,
         pass_env=pass_env,
     )
+
+
+@main.command("requirements", help="""Cache requirements for a venv.""")
+@click.argument("ident", type=str)
+@click.pass_context
+def requirements(ctx, ident):
+    ctx.obj["session"].requirements(
+        ident=ident,
+    )
