@@ -15,6 +15,7 @@ import traceback
 import typing as t
 
 import click
+from packaging.version import Version
 import pexpect
 from rich import print as rich_print
 from rich.pretty import Pretty
@@ -23,6 +24,9 @@ from rich.table import Table
 
 logger = logging.getLogger(__name__)
 
+DEFAULT_RIOT_PATH = ".riot"
+DEFAULT_RIOT_ENV_PREFIX = "venv_py"
+
 SHELL = os.getenv("SHELL", "/bin/bash")
 ENCODING = sys.getdefaultencoding()
 SHELL_RCFILE = """
@@ -148,7 +152,16 @@ class Interpreter:
             py_ex = shutil.which(f"python{self._hint}")
 
         if py_ex:
-            return os.path.abspath(py_ex)
+            # Ensure that we are getting the path of the actual executable,
+            # rather than some wrapping shell script.
+
+            return os.path.abspath(
+                subprocess.check_output(
+                    [py_ex, "-c", "import sys;print(sys.executable)"]
+                )
+                .decode()
+                .strip()
+            )
 
         raise FileNotFoundError(f"Python interpreter {self._hint} not found")
 
@@ -156,22 +169,41 @@ class Interpreter:
     def venv_path(self) -> str:
         """Return the path to the virtual environment for this interpreter."""
         version = self.version().replace(".", "")
-        return os.path.abspath(f".riot/venv_py{version}")
+        env_base_path = os.environ.get("RIOT_ENV_BASE_PATH", DEFAULT_RIOT_PATH)
+        return os.path.abspath(
+            os.path.join(env_base_path, f"{DEFAULT_RIOT_ENV_PREFIX}{version}")
+        )
 
-    def create_venv(self, recreate: bool, path: t.Optional[str] = None) -> str:
-        """Attempt to create a virtual environment for this intepreter."""
+    def exists(self) -> bool:
+        """Return whether the virtual environment for this interpreter exists."""
+        return os.path.isdir(self.venv_path)
+
+    def create_venv(self, recreate: bool, path: t.Optional[str] = None) -> bool:
+        """Attempt to create a virtual environment for this intepreter.
+
+        Returns ``True`` if the virtual environment was created or ``False`` if
+        it already existed.
+        """
         venv_path: str = path or self.venv_path
 
-        if os.path.isdir(venv_path) and not recreate:
-            logger.info(
-                "Skipping creation of virtualenv '%s' as it already exists.", venv_path
-            )
-            return venv_path
+        if os.path.isdir(venv_path):
+            if not recreate:
+                logger.info(
+                    "Skipping creation of virtualenv '%s' as it already exists.",
+                    venv_path,
+                )
+                return False
+            logger.info("Deleting virtualenv '%s'", venv_path)
+            shutil.rmtree(venv_path)
 
         py_ex = self.path()
         logger.info("Creating virtualenv '%s' with interpreter '%s'.", venv_path, py_ex)
-        run_cmd(["virtualenv", f"--python={py_ex}", venv_path], stdout=subprocess.PIPE)
-        return venv_path
+        run_cmd(
+            ["virtualenv", f"--python={py_ex}", venv_path],
+            stdout=subprocess.PIPE,
+        )
+
+        return True
 
 
 @dataclasses.dataclass
@@ -235,16 +267,16 @@ class Venv:
         parent_inst: t.Optional["VenvInstance"] = None,
     ) -> t.Generator["VenvInstance", None, None]:
         # Expand out the instances for the venv.
-        for env_spec in expand_specs(self.env):
+        for env_spec in expand_specs(self.env):  # type: ignore[attr-defined]
             # Bubble up env
             env = parent_inst.env.copy() if parent_inst else {}
             env.update(dict(env_spec))
 
             # Bubble up pys
-            pys = self.pys or [parent_inst.py if parent_inst else None]
+            pys = self.pys or [parent_inst.py if parent_inst else None]  # type: ignore[attr-defined]
 
             for py in pys:
-                for pkgs in expand_specs(self.pkgs):
+                for pkgs in expand_specs(self.pkgs):  # type: ignore[attr-defined]
                     inst = VenvInstance(
                         # Bubble up name and command if not overridden
                         name=self.name or (parent_inst.name if parent_inst else None),
@@ -359,7 +391,12 @@ class VenvInstance:
 
         ident = self.ident
         assert ident is not None, self
-        return "_".join((venv_path, ident))
+        prefix_path = "_".join((venv_path, ident))
+        return (
+            "_".join((venv_path, self.long_hash))[:255]
+            if len(prefix_path) > 255
+            else prefix_path
+        )
 
     @property
     def venv_path(self) -> t.Optional[str]:
@@ -383,8 +420,8 @@ class VenvInstance:
             return None
         return "_".join(
             (
-                f"{rmchars('<=>.,:+@/', n)}{rmchars('<=>.,:+@/', v)}"
-                for n, v in self.pkgs.items()
+                f"{rmchars('<=>.,:+@/', n)}"
+                for n in self.full_pkg_str.replace("'", "").split()
             )
         )
 
@@ -408,9 +445,13 @@ class VenvInstance:
 
         return pip_deps(pkgs)
 
+    @property
+    def long_hash(self) -> str:
+        return hex(hash(self))[2:]
+
     @property
     def short_hash(self) -> str:
-        return hex(hash(self))[2:9]
+        return self.long_hash[:7]
 
     def __hash__(self):
         """Compute a hash for the venv instance."""
@@ -419,6 +460,41 @@ class VenvInstance:
         h.update(self.full_pkg_str.encode())
         return int(h.hexdigest(), 16)
 
+    @property
+    def requirements(self) -> str:
+        """Requirements for dependencies with pinned versions."""
+        # Transform full_pkg_str into requirements.in format
+        pkgs = "\n".join(self.full_pkg_str.replace("'", "").split(" "))
+        _dir = os.path.join(DEFAULT_RIOT_PATH, "requirements")
+        os.makedirs(_dir, exist_ok=True)
+        in_path = os.path.join(_dir, "{}.in".format(self.short_hash))
+        subprocess.check_output(
+            [self.py.path(), "-m", "pip", "install", "pip-tools"],
+        )
+        cmd = [
+            self.py.path(),
+            "-m",
+            "piptools",
+            "compile",
+            "-q",
+            "--no-annotate",
+            in_path,
+        ]
+        if tuple([int(v) for v in self.py.version().strip("()").split(".")]) >= (3, 7):
+            cmd.append("--resolver=backtracking")
+        logger.info(
+            "Compiling requirements file %s at %s.",
+            in_path,
+            self.prefix,
+        )
+        with open(in_path, "w+b") as f:
+            f.write(pkgs.encode("utf-8"))
+            f.flush()
+
+            out = subprocess.check_output(cmd)
+
+            return out.decode("utf-8")
+
     @property
     def bin_path(self) -> t.Optional[str]:
         prefix = self.prefix
@@ -497,17 +573,25 @@ class VenvInstance:
         py: t.Optional[Interpreter] = None,
         recreate: bool = False,
         skip_deps: bool = False,
+        recompile_reqs: bool = False,
+        child_was_installed: bool = False,
     ) -> None:
         # Propagate the interpreter down the parenting relation
         self.py = py = py or self.py
+        if recompile_reqs:
+            recreate = True
 
-        # We only install dependencies if the prefix directory does not
-        # exist already. If it does exist, we assume it is in a good state.
+        exists = self.prefix is not None and os.path.isdir(self.prefix)
+
+        installed = False
         if (
             py is not None
             and self.pkgs
             and self.prefix is not None
-            and not os.path.isdir(self.prefix)
+            # We only install dependencies if the prefix directory does not
+            # exist already. If it does exist, we assume it is in a good state.
+            and (not os.path.isdir(self.prefix) or recreate or recompile_reqs)
+            and not child_was_installed
         ):
             venv_path = self.venv_path
             assert venv_path is not None, py
@@ -519,15 +603,24 @@ class VenvInstance:
 
             pkg_str = self.pkg_str
             assert pkg_str is not None
+            compiled_requirements_file = (
+                f"{DEFAULT_RIOT_PATH}/requirements/{self.short_hash}.txt"
+            )
+            if recompile_reqs or not os.path.exists(compiled_requirements_file):
+                _ = self.requirements
+            cmd = (
+                f"pip --disable-pip-version-check install --prefix '{self.prefix}' --no-warn-script-location "
+                f"-r {compiled_requirements_file}"
+            )
             logger.info(
                 "Installing venv dependencies %s at %s.",
-                pkg_str,
+                compiled_requirements_file,
                 self.prefix,
             )
             try:
                 Session.run_cmd_venv(
                     venv_path,
-                    f"pip --disable-pip-version-check install --prefix '{self.prefix}' --no-warn-script-location {pkg_str}",
+                    cmd,
                     env=env,
                 )
             except CmdFailure as e:
@@ -535,9 +628,13 @@ class VenvInstance:
                     f"Failed to install venv dependencies {pkg_str}\n{e.proc.stdout}",
                     e.proc,
                 )
+            else:
+                installed = True
 
         if not self.created and self.parent is not None:
-            self.parent.prepare(env, py)
+            self.parent.prepare(
+                env, py, child_was_installed=installed or exists or child_was_installed
+            )
 
 
 @dataclasses.dataclass
@@ -618,6 +715,7 @@ class Session:
         pythons: t.Optional[t.Set[Interpreter]] = None,
         skip_missing: bool = False,
         exit_first: bool = False,
+        recompile_reqs: bool = False,
     ) -> None:
         results = []
 
@@ -688,7 +786,12 @@ class Session:
                 }
             )
 
-            inst.prepare(env, recreate=recreate_venvs, skip_deps=skip_base_install)
+            inst.prepare(
+                env,
+                recreate=recreate_venvs,
+                skip_deps=skip_base_install,
+                recompile_reqs=recompile_reqs,
+            )
 
             pythonpath = inst.pythonpath
             if pythonpath:
@@ -778,10 +881,19 @@ class Session:
             sys.exit(1)
 
     def list_venvs(
-        self, pattern, venv_pattern, pythons=None, out=sys.stdout, pipe_mode=False
+        self,
+        pattern,
+        venv_pattern,
+        pythons=None,
+        out=sys.stdout,
+        pipe_mode=False,
+        interpreters=False,
+        hash_only=False,
     ):
+        python_interpreters = set()
+        venv_hashes = set()
         table = None
-        if not pipe_mode:
+        if not (pipe_mode or interpreters or hash_only):
             table = Table(
                 "No.",
                 "Hash",
@@ -803,6 +915,11 @@ class Session:
                 continue
             pkgs_str = inst.full_pkg_str
             env_str = env_to_str(inst.env)
+            if interpreters or hash_only:
+                python_interpreters.add(inst.py._hint)
+                venv_hashes.add(inst.short_hash)
+                continue
+
             if pipe_mode:
                 print(
                     f"[#{n}]  {inst.short_hash}  {inst.name:12} {env_str} {inst.py} Packages({pkgs_str})"
@@ -810,7 +927,7 @@ class Session:
             else:
                 table.add_row(
                     f"[cyan]#{n}[/cyan]",
-                    f"[bold cyan]{hex(hash(inst))[2:9]}[/bold cyan]",
+                    f"[bold cyan]{inst.short_hash}[/bold cyan]",
                     f"[bold]{inst.name}[/bold]",
                     Pretty(inst.py),
                     env_str or "--",
@@ -820,6 +937,12 @@ class Session:
         if table:
             rich_print(table)
 
+        elif hash_only and venv_hashes:
+            print("\n".join(sorted(venv_hashes)))
+
+        elif interpreters and python_interpreters:
+            print("\n".join(sorted(python_interpreters, key=Version)))
+
     def generate_base_venvs(
         self,
         pattern: t.Pattern[str],
@@ -848,27 +971,30 @@ class Session:
 
         for py in required_pys:
             try:
-                venv_path = py.create_venv(recreate)
+                # We check if the venv existed already. If it didn't, we know we
+                # have to install the dev package. Otherwise we assume that it
+                # already has the dev package installed.
+                install_deps = py.create_venv(recreate)
             except CmdFailure as e:
                 logger.error("Failed to create virtual environment.\n%s", e.proc.stdout)
             except FileNotFoundError:
                 logger.error("Python version '%s' not found.", py)
             else:
-                if skip_deps:
+                if not install_deps and skip_deps:
                     logger.info("Skipping global deps install.")
                     continue
 
                 # Install the dev package into the base venv.
-                install_dev_pkg(venv_path)
+                install_dev_pkg(py.venv_path)
 
     def _generate_shell_rcfile(self):
         with tempfile.NamedTemporaryFile() as rcfile:
             rcfile.write()
             rcfile.flush()
 
-    def shell(self, ident, pass_env):
+    def _venvs_matching_identifier(self, identifier):
         for n, inst in enumerate(self.venv.instances()):
-            if ident != f"#{n}" and not hex(hash(inst))[2:].startswith(ident):
+            if identifier != f"#{n}" and not inst.long_hash.startswith(identifier):
                 continue
 
             assert inst.py is not None, inst
@@ -876,7 +1002,15 @@ class Session:
                 venv_path = inst.venv_path
             except FileNotFoundError:
                 raise RuntimeError("%s not available" % inst.py)
+            yield inst, venv_path
 
+    def requirements(self, ident):
+        for inst, _ in self._venvs_matching_identifier(ident):
+            with Status("Producing requirements.txt"):
+                _ = inst.requirements
+
+    def shell(self, ident, pass_env):
+        for inst, venv_path in self._venvs_matching_identifier(ident):
             logger.info("Launching shell inside venv instance %s", inst)
             logger.debug("Setting venv path to %s", venv_path)
 
@@ -1052,6 +1186,13 @@ def pip_deps(pkgs: t.Dict[str, str]) -> str:
 
 
 def install_dev_pkg(venv_path):
+    for setup_file in {"setup.py", "pyproject.toml"}:
+        if os.path.exists(setup_file):
+            break
+    else:
+        logger.warning("No Python setup file found. Skipping dev package installation.")
+        return
+
     logger.info("Installing dev package (edit mode) in %s.", venv_path)
     try:
         Session.run_cmd_venv(
