@@ -74,10 +74,11 @@ Options:
   --help           Show this message and exit.
 
 Commands:
-  generate  Generate base virtual environments.
-  list      List all virtual env instances matching a pattern.
-  run       Run virtualenv instances with names matching a pattern.
-  shell     Launch a shell inside a venv.
+  generate      Generate base virtual environments.
+  list          List all virtual env instances matching a pattern.
+  requirements  Cache requirements for a venv.
+  run           Run virtualenv instances with names matching a pattern.
+  shell         Launch a shell inside a venv.
 """.lstrip()
     )
     assert result.stderr == ""
@@ -167,10 +168,11 @@ Options:
   --help           Show this message and exit.
 
 Commands:
-  generate  Generate base virtual environments.
-  list      List all virtual env instances matching a pattern.
-  run       Run virtualenv instances with names matching a pattern.
-  shell     Launch a shell inside a venv.
+  generate      Generate base virtual environments.
+  list          List all virtual env instances matching a pattern.
+  requirements  Cache requirements for a venv.
+  run           Run virtualenv instances with names matching a pattern.
+  shell         Launch a shell inside a venv.
 """.lstrip()
     )
     assert result.stderr == ""
@@ -456,7 +458,7 @@ def test_failure():
     assert 1 == 0
 """,
     )
-    result = tmp_run("riot run -s pass")
+    result = tmp_run("riot --pipe run -s pass")
     assert result.returncode == 0, result.stderr
     assert re.search(
         r"""
@@ -474,7 +476,10 @@ test_success.py .*
 1 passed with 0 warnings, 0 failed\n""".lstrip(),
         result.stdout,
     ), result.stdout
-    assert result.stderr == ""
+    assert (
+        "No Python setup file found. Skipping dev package installation."
+        in result.stderr
+    )
 
     result = tmp_run("riot run -s fail")
     assert re.search(
@@ -522,9 +527,12 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot run -s test_cmdargs -- -k filter")
+    result = tmp_run("riot --pipe run -s test_cmdargs -- -k filter")
     assert "cmdargs=-k filter" not in result.stdout
-    assert result.stderr == ""
+    assert (
+        "No Python setup file found. Skipping dev package installation."
+        in result.stderr
+    )
     assert result.returncode == 0
 
     rf_path.write_text(
@@ -555,10 +563,12 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot run test")
-    assert result.returncode == 1
-    assert "setup.py" in result.stderr, result.stderr
-    assert "Dev install failed, aborting!" in result.stderr, result.stderr
+    result = tmp_run("riot --pipe run test")
+    assert result.returncode == 0
+    assert (
+        "No Python setup file found. Skipping dev package installation."
+        in result.stderr
+    )
 
 
 def test_bad_interpreter(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
@@ -639,7 +649,7 @@ venv = Venv(
 
     version = "".join((str(_) for _ in sys.version_info[:3]))
 
-    venv_name = f"venv_py{version}_pytest"
+    venv_name = f"venv_py{version}_pip_pytest"
     parent_venv_name = f"venv_py{version}_pip"
     py_dot_version = ".".join((str(_) for _ in sys.version_info[:2]))
 
@@ -698,7 +708,7 @@ venv = Venv(
     env = dict(_.split("=", maxsplit=1) for _ in result.stdout.splitlines() if "=" in _)
     assert result.returncode == 0, result.stderr
 
-    venv_name = "venv_py{}_pytest".format(
+    venv_name = "venv_py{}_pip_pytest".format(
         "".join((str(_) for _ in sys.version_info[:3]))
     )
     parent_venv_name = "venv_py{}_pip".format(
@@ -773,7 +783,7 @@ venv = Venv(
 """,
     )
     result = tmp_run("riot -Pv run -s child")
-    venv_path = tmp_path / ".riot/venv_py{}_pytest_pip".format(
+    venv_path = tmp_path / ".riot/venv_py{}_pip_pytest".format(
         "".join((str(_) for _ in sys.version_info[:3]))
     )
     assert f"Creating virtualenv '{venv_path}'" in result.stderr, result.stderr
@@ -863,3 +873,94 @@ setup(
         r"Setting venv path to .+[.]riot/venv_py[0-9]+_requests", result.stderr
     )
     assert result.returncode == 0, result.stderr
+
+
+def test_long_path(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
+    wd = tmp_path / ("a" * (230 - len(str(tmp_path))))
+    wd.mkdir()
+
+    rf_path = wd / "riotfile.py"
+    rf_path.write_text(
+        """
+from riot import Venv, latest
+venv = Venv(
+    name="test",
+    pys=["3"],
+    pkgs={"requests": latest},
+    command="python -c 'import requests;print(\\\"TEST OK\\\")'",
+)
+""",
+    )
+
+    result = tmp_run("riot -vd --pipe run test", cwd=wd)
+
+    assert result.returncode == 0, result.stderr
+
+    assert re.search(r"venv_py[0-9]+_requests", result.stderr) is None
+    assert "TEST OK" in result.stdout
+
+
+def test_requirements_lockfile_created(
+    tmp_path: pathlib.Path, tmp_run: _T_TmpRun
+) -> None:
+    rf_path = tmp_path / "riotfile.py"
+    rf_path.write_text(
+        """
+from riot import Venv
+venv = Venv(
+    pys=[3],
+    pkgs={"requests": ""},
+    name="test",
+    command="env",
+    venvs=[Venv(pkgs={"pytest": ""})]
+)
+""",
+    )
+    _hash = tmp_run("riot list --hash-only").stdout.strip("\n")
+    result = tmp_run("riot requirements {}".format(_hash))
+    assert result.returncode == 0, result.stderr
+
+    lockfile_path = str(
+        tmp_path
+        / os.path.join(
+            ".riot",
+            "requirements",
+            "{}.txt".format(_hash),
+        )
+    )
+    assert os.path.exists(lockfile_path)
+    with open(lockfile_path, "r") as f:
+        lines = f.readlines()
+        assert "==" in lines[-1]
+        assert "#" in lines[0]
+
+
+def test_recompile_flag(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
+    rf_path = tmp_path / "riotfile.py"
+    rf_path.write_text(
+        """
+from riot import Venv
+venv = Venv(
+    pys=[3],
+    pkgs={"requests": ""},
+    name="test",
+    command="env",
+    venvs=[Venv(pkgs={"pytest": ""})]
+)
+""",
+    )
+    _hash = tmp_run("riot list --hash-only").stdout.strip("\n")
+    lockfile_path = str(
+        tmp_path
+        / os.path.join(
+            ".riot",
+            "requirements",
+            "{}.txt".format(_hash),
+        )
+    )
+    assert not os.path.exists(lockfile_path)
+
+    result = tmp_run("riot run -c test")
+    assert result.returncode == 0, result.stderr
+
+    assert os.path.exists(lockfile_path)
